// @dart = 2.12

import 'package:flutter/material.dart';
import 'package:smooth_ui_library/smooth_ui_library.dart';

void main() {
  runApp(
    const MaterialApp(
      debugShowCheckedModeBanner: false,
      home: SafeArea(
        child: ExampleApp(),
      ),
    ),
  );
}

class Fruit implements Comparable<Fruit> {
  const Fruit(this.name);
  final String name;

  @override
  int compareTo(Fruit other) => name.compareTo(other.name);

  @override
  String toString() => name;
}

class FruitCategory extends Category<Fruit> {
  FruitCategory(Fruit value, [Iterable<FruitCategory>? children])
      : super(value, children);

  @override
  FruitCategory? operator [](Fruit childValue) {
    return super[childValue] as FruitCategory?;
  }

  @override
  String get label => value.name;
}

FruitCategory categories = FruitCategory(
  const Fruit('fruit'),
  <FruitCategory>{
    FruitCategory(
      const Fruit('apple'),
      <FruitCategory>{
        FruitCategory(
          const Fruit('red'),
          <FruitCategory>[
            FruitCategory(const Fruit('Red Delicious')),
            FruitCategory(const Fruit('Fuji')),
            FruitCategory(const Fruit('Crispin')),
            FruitCategory(const Fruit('Pink Lady')),
          ],
        ),
        FruitCategory(
          const Fruit('yellow'),
          <FruitCategory>[
            FruitCategory(const Fruit('Yellow Delicious')),
            FruitCategory(const Fruit('Ginger Gold')),
          ],
        ),
        FruitCategory(
          const Fruit('green'),
          <FruitCategory>[
            FruitCategory(const Fruit('Granny Smith')),
          ],
        ),
      },
    ),
    FruitCategory(
      const Fruit('berry'),
      <FruitCategory>{
        FruitCategory(const Fruit('blueberry')),
        FruitCategory(const Fruit('raspberry')),
      },
    ),
  },
);

FruitCategory? getCategory(Iterable<Fruit> path) {
  if (path.isEmpty) {
    return null;
  }
  FruitCategory? result = categories.value == path.first ? categories : null;
  final List<Fruit> followPath = path.skip(1).toList();
  while (result != null && followPath.isNotEmpty) {
    result = result[followPath.first];
    followPath.removeAt(0);
  }
  return result;
}

class ExampleApp extends StatefulWidget {
  const ExampleApp();

  @override
  State<ExampleApp> createState() => _ExampleAppState();
}

class _ExampleAppState extends State<ExampleApp> {
  Set<Fruit> currentCategories = <Fruit>{
    const Fruit('raspberry'),
    const Fruit('Fuji')
  };
  List<Fruit> currentCategoryPath = <Fruit>[
    const Fruit('fruit'),
    const Fruit('apple'),
  ];

  Widget _addCategoryDialog(BuildContext context, FruitCategory parent) {
    final TextEditingController controller = TextEditingController();
    void addCategory(String name) {
      Navigator.of(context)
          .pop(name.isNotEmpty ? FruitCategory(Fruit(name)) : null);
    }

    return AlertDialog(
      content: TextField(
        autofocus: true,
        controller: controller,
        decoration: const InputDecoration(
          hintText: 'Enter new category name',
        ),
        onSubmitted: addCategory,
      ),
      actions: <Widget>[
        TextButton(
          child: const Text('OK'),
          onPressed: () => addCategory(controller.text),
        ),
        TextButton(
          child: const Text('Cancel'),
          onPressed: () => Navigator.of(context).pop(),
        ),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return Theme(
      data: Theme.of(context).copyWith(
        backgroundColor: Colors.lightGreenAccent,
        canvasColor: Colors.lightGreenAccent,
        scaffoldBackgroundColor: Colors.lightGreenAccent,
        floatingActionButtonTheme: const FloatingActionButtonThemeData(
          backgroundColor: Colors.green,
          foregroundColor: Colors.black,
        ),
        checkboxTheme: CheckboxTheme.of(context).copyWith(
          fillColor:
              MaterialStateColor.resolveWith((Set<MaterialState> states) {
            if (states.contains(MaterialState.selected)) {
              return Colors.green;
            }
            return Colors.black38;
          }),
        ),
        chipTheme:
            ChipTheme.of(context).copyWith(backgroundColor: Colors.green),
      ),
      child: Scaffold(
        body: SmoothCategoryPicker<Fruit>(
          categoryFinder: getCategory,
          currentPath: currentCategoryPath,
          currentCategories: currentCategories,
          onCategoriesChanged: (Set<Fruit> value) {
            debugPrint('Categories: ${value.join(', ')}');
            setState(() {
              currentCategories = value;
            });
          },
          onPathChanged: (Iterable<Fruit> path) {
            setState(() {
              currentCategoryPath = path.toList();
            });
          },
          onAddCategory: (Iterable<Fruit> path) {
            final FruitCategory? currentCategory = getCategory(path);
            if (currentCategory != null) {
              showDialog<FruitCategory>(
                      builder: (BuildContext context) =>
                          _addCategoryDialog(context, currentCategory),
                      context: context)
                  .then<void>((FruitCategory? category) {
                if (category != null) {
                  setState(() {
                    // Remove the parent from the set of assign categories,
                    // since it isn't a leaf anymore.
                    currentCategories.remove(currentCategory.value);
                    currentCategory.add(category);
                    // If they added a new category, they must mean that the
                    // category applies.
                    currentCategories.add(category.value);
                  });
                }
              });
            }
          },
        ),
      ),
    );
  }
}
